# Tutorial Overview
This tutorial covers the basics of using [DVC](https://dvc.org) and DAGsHub to create version-controlled data science projects,
as well as tracking experiments with the [DAGsHub Logger](/docs/reference/open_data_science_formats).

You will learn about data exploration, tracking experiment parameters and metrics, comparing experiments, and more...

## Detecting questions about Machine Learning
In this tutorial, we'll create a model to predict whether a question on the
[Cross Validated Stack Exchange](https://stats.stackexchange.com/) concerns Machine Learning or not.

This kind of prediction can be useful if we want to recommend to a user to add the `machine-learning` tag to their question for example,
which can make it more likely they will get an answer.

This task is simple and clean enough for a tutorial but leaves room for experimentation with feature engineering,
data enrichment, and model selection.

The tutorial is divided into several "levels", each of which demonstrates another workflow improvement.
It's designed so that you learn something useful at each "level", even if the level after that is less to your liking, and you
choose to stop early.

The levels are:

0. [Data Exploration](/docs/experiment-tutorial/0-data) - Getting the data and trying to understand it, otherwise known as **doing exploratory data analysis**.
1. [Setup](/docs/experiment-tutorial/1-setup) - Creating a DAGsHub account and project.
1. [Data Versioning](/docs/experiment-tutorial/2-data-versioning) - Using [DVC](https://dvc.org) to keep track of data and model versions.
1. [Experimentation](/docs/experiment-tutorial/3-experiments) - Logging hyperparameters and metrics to DAGsHub to keep track of and [comparing different experiments](/docs/collaborating_on_dagshub/discovering_experiments).
<br/>
<br/>
<center>![Screenshot](assets/pie_question.png)
<sub>Delicious statistics 😋 (source: [Cross Validated](https://stats.stackexchange.com/questions/423/what-is-your-favorite-data-analysis-cartoon/694#694))</sub></center>

## Too slow for you?
Here is a link to the complete code repo. You can go over it or use the code as you wish.

[https://dagshub.com/DAGsHub-Official/DAGsHub-Tutorial](https://dagshub.com/DAGsHub-Official/DAGsHub-Tutorial)

The tutorial will guide you, step-by-step, to create this repo.
