# Setup
## Section overview
The first section of this tutorial covers setting up our project.
This includes the following tasks:

* Creating an account and repo in DAGsHub
* Installing DVC as well as the optional installation needed to use
remote caches for a project (we assume you have Git installed and are familiar with it to some extent).
* Project initialization in Git and DVC
* Creating a virtualenv and installing the needed requirements for the first part of the project

If you are familiar with these steps, you can skip to the next part which covers [pipeline creation](pipeline.md).
## Joining DAGsHub...
...is really easy. Just [sign up](/user/sign_up).
Then, after logging in, create a new repo, simply by clicking on the plus sign and create repository in the
 nav bar.
<br/>
<br/>
<center>![Screenshot](assets/create_repo.png)
<sub>Repo creation button</sub></center>
<br/>
This opens up a dialog, which should be somewhat familiar, in which you can set the repository name,
description and other options.
<br/>
<br/>
<center>![Screenshot](assets/create_mnist_repo.png)
<sub>Repo creation dialog</sub></center>
<br/>
For this tutorial, fill in the Name and description, and leave everything
else with the default settings.

!!! info
    Note the **Local DVC cache URL** setting, which is unique to DAGsHub. We will get back to it later in the tutorial.

 Done with repo creation. On to project initialization.
## Installing DVC
We need to make sure we have Git and DVC installed on our computer.

We won't cover Git installation here, but if you need it, here is a link to an [installation guide](https://git-scm.com/book/en/v2/Getting-Started-Installing-Git){target=_blank}.

**This tutorial assumes a Python 3 installation.** <br/>
Make sure you aren't using Python 2 by mistake, using `python --version`.

For full documentation and all options for installation of DVC, [see the official docs](https://dvc.org/doc/install){target=_blank}.

DVC is actually a python module. You can run it from your OS environment or from your currently
activated python environment. As such, the fastest way to install DVC is using pip. Just open a terminal and type:
```
pip install dvc
```
and voilà. Done.

!!! warning
    This tutorial was last updated to **DVC version 1.1.7** .
    If you are using an older version, please update.
    If you are using a newer version, be aware that the behavior of some commands may change.

## Setting up our project
Create a directory name "dagshub_mnist" for the project somewhere on your computer.
Open a terminal and input the following:
```bash
cd path/to/folder/dagshub_mnist

git init
dvc init
```

The last command initializes the project as a DVC project, which sets up the following directory structure:
```text
.
├── .dvc
│   ├── .gitignore
│   ├── config
│   ├── plots
│   │   ├── confusion.json
│   │   ├── default.json
│   │   ├── scatter.json
│   │   └── smooth.json
│   └── tmp
│       └── index
└── .git
    ┊
```

This is somewhat similar to the .git folder contained in every git repo, except some of its contents will be tracked
using git.

* `.dvc/config` is similar to `.git/config`. By default it's empty. More on this later on.
* `.dvc/.gitignore` makes sure git ignores dvc internal files that shouldn't be tracked by git.
* `.dvc/plots` contains predefined templates for plots you can generate using `dvc` - more info [here](https://dvc.org/doc/command-reference/plots){target=_blank}.
* `.dvc/tmp` is used by dvc to store temporary files, this shouldn't interest the average user.
* `.dvc/cache` doesn't exist yet, but it is where DVC will keep the different versions of our data files. It's very similar in principle to `.git/objects`.

Now, we will set the remote to our repo on [dagshub.com](/){target=_blank}. This can be done using the
following command:
```bash
git remote add origin https://dagshub.com/<username>/<repo-name>.git
```
Finally, let's create 3 folders, for each of our main project components.
```bash
mkdir code data metrics
```
This will add a clear structure for the project.
## Creating a virtual environment and installing requirements
We strongly recommend using a virtual environment to run the code in this tutorial. You may use the environment manager
of your choosing. In this example we will use `virtualenv`.

In the project directory type the following commands:

=== "Linux/Mac"
    ```bash
    virtualenv venv
    echo venv/ >> .gitignore
    source venv/bin/activate
    ```
=== "Windows"
    ```batch
    virtualenv venv
    echo venv/ >> .gitignore
    venv\Scripts\activate.bat
    ```

The first command makes sure the virtual environment packages are not tracked by Git.<br/>
The second command activates our virtual python environment. This ensures that no python package we use contaminates
our global python installation. <br/>
**The rest of this tutorial should be executed using this environment ** <br/>
If you exit the shell session, or want to create another, **make sure to activate the virtual environment in that shell session first**.

### Installing requirements
To install the requirements for the first part of this project, simply download the
[requirements.txt](/DAGsHub-Official/DAGsHub-Tutorial-MNIST/raw/92c319cd1a14c8bc6c72a7de34fd78448aa021fd/requirements.txt)
into your project folder.

!!! tip
    Alternatively, you can create a file called _**requirements.txt**_ and copy the following into it:
    ```text
    numpy==1.15.4
    pandas==0.23.4
    python-dateutil==2.7.5
    pytz==2018.7
    scikit-learn==0.20.2
    scipy==1.2.0
    six==1.12.0
    sklearn==0.0
    ```

Now, to install them type:
```bash
pip install -r requirements.txt
```

## Committing progress to Git
Let's check the Git status of our project:
```bash
$ git status -s
A  .dvc/.gitignore
A  .dvc/config
A  .dvc/plots/confusion.json
A  .dvc/plots/default.json
A  .dvc/plots/scatter.json
A  .dvc/plots/smooth.json
?? .gitignore
?? requirements.txt
```

Now let's commit this to Git using the command line:
```bash
git add .
git commit -m "Initialized project"
git push -u origin master
```
The last command might request your DAGsHub.com username and password.

## What changed?
Let's open up our repo on DAGsHub. It should look something like this:
<br/>
<br/>
<center>![Screenshot](assets/first_commit_repo.png)
<sub>Repo view after first commit</sub></center>
<br/>
The top part should look familiar to anyone that has used a Git server before. However, notice the bottom
window. This is a new addition in DAGsHub, and will, later on in this tutorial, **show our data pipeline**.

In the next section, we will use dvc to define our pipeline. That is where the magic happens.
